﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;
using System.Net.Sockets;

namespace V409Control
{
    public delegate void DisconnectDelegate(Boolean reConnect);  //该地方存在疑惑
    public class Equipment
    {

        public Boolean uUart = true;  //是否使用串口
        public SerialPort serialPort1;
        private TcpClient tcpClient;   //实际上这里定义了TCP类的两个成员变量
                                       //    private TcpClient server;

        private string netIpStr = "192.168.1.192";
        private int netPort = 23;           //网络相关设置

        private string comNameStr = null;
       
        private int comBps = 9600;
        public NetworkStream clientStream = null;        //这个地方有两种不同的方式可以实现通过网络发送数据
                                                         //   Socket client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);//这是第二种方法
                                                         //****IPV4,流式socket方式，TCP协议传输数据****//

        public event DisconnectDelegate DisconnectEvent;  //这个不是委托事件吗

    
        private int heartBeatReTries = 0;
        private int heartBeatMaxTryNum = 10;
     
        public int ComBps
        {
            set { comBps = value; }
            get { return comBps; }
        }
        public string NetIpStr
        {
            set
            {
                if (value.Length > 6)
                {
                    netIpStr = value;
                }
            }
            get { return netIpStr; }
        }
        public int NetPort
        {
            set
            {
                if (value > 0)
                {
                    netPort = value;
                }
            }
            get { return netPort; }
        }
        public Boolean UUart
        {
            set { uUart = value; }
            get { return uUart; }
        }
        public string ComNameStr
        {
            set
            {
                if (value == "")
                {
                    comNameStr = null;
                }
                else
                    comNameStr = value;
            }
            get { return comNameStr; }
        }
        public int HeartBeatReTries
        {
            set
            {
                heartBeatReTries = value;
            }
            get
            {
                return heartBeatReTries;
            }
        }
        public int HeartBeatMaxTryNum
        {
            set
            {
                heartBeatMaxTryNum = value;
            }
            get
            {
                return heartBeatMaxTryNum;
            }
        }
        public Equipment()
        {
            serialPort1 = new SerialPort();
            tcpClient = new TcpClient();

        }
        public byte[] HeartBeat = new byte[8] { 0xd6, 0x00, 0x00, 0x00, 0x00, 0xff, 0x00, 0xaa };
        public char[] buf = new char[30];
        public Boolean readflag = false;
        public byte[] AckPktBuf = new byte[8];
        //******连接设备的方法*****//
        public void EquipmentConnect()
        {

             if (uUart)   //直接定位到这里，，没有因为按钮的变化发生改变
             {
                 try
                 {
                     if ((!serialPort1.IsOpen) && (ComNameStr != null))//知道串口没打开，并且串口号不为零，调试的结果发现，直接跳过了这里
                     {
                         serialPort1.BaudRate = ComBps;
                         serialPort1.PortName = comNameStr;  //配置
                         serialPort1.Open();
                     }
                 }
                 catch (Exception e)
                 {
                     MessageBox.Show(e.Message);
                 }

             }
            else
             {

                 if (clientStream == null)
                 {
                     try
                     {
                         tcpClient = new TcpClientWithTimeout(netIpStr, netPort, 500).Connect();
                     }
                     catch (Exception e)
                     {
                         MessageBox.Show(e.Message);

                     }
                     if (tcpClient.Connected)
                     {
                         clientStream = tcpClient.GetStream();
                     }
                 }
             }
          
        }
        public void WriteByteToSerialPort(byte data)                   //单字节写入串口
        {
            byte[] Buffer = new byte[1] { data };                 //定义数组
            if (serialPort1.IsOpen)                                     //传输数据的前提是端口已打开
            {
                try
                {
                    serialPort1.Write(Buffer, 0, 1);                    //写数据
                }
                catch
                {
                    MessageBox.Show("串口数据发送出错，请检查.", "错误");//错误处理

                }
            }
        }
        //*****接收数据的方法*****//
        public void ReceiveData()
        {
            //  int num;
            if (clientStream != null)
            {
                if (clientStream.DataAvailable)
                {
                    clientStream.Read(AckPktBuf, 0, 8);
                    readflag = true;
                }
            }
        }
        public enum CmdType
        {
            HeartBeat = 0xaa,
            SaveCfg = 0x1a,
            RecoveryCfg = 0x1b,
            SetSelectedNo = 0xfb,
            PowerEn = 0xfd,
            SetFrameSize = 0x5a,
            Login = 0x6a,
            Register = 0x6b,
        };
        public enum CmdType_a
        {
            pin_jie = 0x01,
            Rotate_90 = 0x02,
            Rotate_180 = 0x03,
            Muil_Mode = 0x04,
            Source_Mode = 0x05,
            Pip_Loc = 0x06,
            Pip_Size = 0x07,
            Dhcp = 0x08,

        };
        //****发送数据的方法*****//	
        public Boolean EquipmentSend(byte[] buf)
        {
            if (!GetEquipmentStat())        //检测通信通道有没有打开，该函数主要传出一个布尔值
            {
                MessageBox.Show("请打开连接，再进行操作", "温馨提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }
            if (clientStream != null)
            {
                if (clientStream.CanWrite)
                {
                    clientStream.Write(buf, 0, buf.Length);
                    //   client.Send(buf);
                }
                else
                {
          
                    MessageBox.Show("无法发送数据,请检查网络连接是否已断开", "温馨提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    EquipmentClose();
                    DisconnectEvent(true);
                    return false;
                }
            }
            else
            {
                DisconnectEvent(false);
                return false;
            }
            return true;
        }

        //****发送数据的方法二*****//

        public void EquipmentSend(string str)
        {
            if (clientStream != null)
            {
                if (clientStream.CanWrite)
                {
                    byte[] buf;
                    buf = Encoding.ASCII.GetBytes(str);
                    clientStream.Write(buf, 0, buf.Length);
                }
                else
                {
                    MessageBox.Show("无法写入数据流,请检查网络连接是否已断开。");
                    EquipmentClose();
                    DisconnectEvent(true);
                }
            }
            else
            {
                DisconnectEvent(false);
            }
        }


        //****关闭设备的方法****//
        public void EquipmentClose()
        {
            if (serialPort1.IsOpen)
            {
                serialPort1.Close();
                ComNameStr = null;
            }
            if (clientStream != null)
            {
                tcpClient.Close();
                clientStream.Close();
                clientStream = null;
            }
        }

        //****检测通信通道有没有打开的方法****//	
        public Boolean GetEquipmentStat()
        {
            if (uUart)
            {
                if (serialPort1.IsOpen)
                    return true;
                else
                    return false;
            }
            if (clientStream == null)
                return false;
            else
                return true;
        }
    }
}
